
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#pragma once

#include <set>

#include "maps.h"
#include "matrix.h"
#include "row.h"

namespace algorithm
{
   /// returns the unique facet class representative. The class is generated with the provided maps.
   /// Precondition: input facet must be normalized.
   template <typename Integer>
   Facet<Integer> facetClassRepresentative(const Facet<Integer>&, const Maps&);
   /// returns the unique vertex class representative. The class is generated with the provided maps.
   template <typename Integer>
   Vertex<Integer> vertexClassRepresentative(const Vertex<Integer>&, const Maps&);
   /// Creates a set of facets which is the complete facet class containing the input facet.
   /// Precondition: input facet must be normalized.
   template <typename Integer>
   std::set<Facet<Integer>> facetClass(const Facet<Integer>&, const Maps&);
   /// Creates a set of vertices which is the complete vertex class containing the input vertex.
   template <typename Integer>
   std::set<Vertex<Integer>> vertexClass(const Vertex<Integer>&, const Maps&);
   /// Reduces a list of facets to just the representatives.
   /// Precondition: input facets must be normalized.
   template <typename Integer>
   Facets<Integer> facetClasses(Facets<Integer>, const Maps&);
   /// Reduces a list of vertices to just the representatives.
   template <typename Integer>
   Vertices<Integer> vertexClasses(Vertices<Integer>, const Maps&);
   /// Reduces a list of facets to just the representatives.
   /// Precondition: input facets must be normalized.
   template <typename Integer>
   Facets<Integer> facetClasses(std::set<Facet<Integer>>, const Maps&);
   /// Reduces a list of vertices to just the representatives.
   template <typename Integer>
   Facets<Integer> vertexClasses(std::set<Facet<Integer>>, const Maps&);
}

#include "algorithm_classes.eti"

