
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#include "input_equation.h"

#include <stdexcept>
#include <string>
#include <tuple>

#include "algorithm_row_operations.h"
#include "constraint_type.h"
#include "input_common.h"
#include "input_inequality_named_variables.h"
#include "input_inequality_unnamed_variables.h"
#include "input_keywords.h"
#include "istream_peek_line.h"

namespace
{
   std::pair<Equation<int>, ConstraintType> equation(std::istream&, const Names&);
}

Equations<int> input::implementation::equations(std::istream& stream, const Names& names)
{
   Equations<int> equations;
   std::string token;
   if ( !std::getline(stream, token) )
   {
      throw std::invalid_argument("Cannot read equations: file is at an invalid position.");
   }
   token = trimWhitespace(token);
   if ( !isKeywordEquations(token) )
   {
      throw std::invalid_argument("Cannot read equations: file is at an invalid position.");
   }
   while ( stream && !isKeyword(trimWhitespace(peekLine(stream))) )
   {
      if ( trimWhitespace(peekLine(stream)).empty() )
      {
         std::getline(stream, token);
         continue;
      }
      Equation<int> eq;
      ConstraintType type;
      std::tie(eq, type) = equation(stream, names);
      if ( !eq.empty() )
      {
         if ( type != ConstraintType::Equation )
         {
            throw std::invalid_argument("Cannot use an inequality in the system of equations.");
         }
         equations.push_back(eq);
      }
      while ( stream && stream.peek() != '\n' )
      {
         stream.get();
      }
      stream.get();
   }
   if ( stream && isKeywordEnd(firstWord(peekLine(stream))) )
   {
      std::getline(stream, token);
   }
   return equations;
}

namespace
{
   std::pair<Equation<int>, ConstraintType> equation(std::istream& stream, const Names& names)
   {
      if ( names.empty() || !input::containsNames(names, stream) )
      {
         return input::implementation::inequality(stream);
      }
      else
      {
         return input::implementation::inequality(stream, names);
      }
   }
}

