
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#include "integer_type_detection.h"

#include <cstring>
#include <stdexcept>
#include <string>

namespace
{
   /// Returns the integer type interpreted from char*.
   IntegerType integerTypeFromString(char*);
}

IntegerType integerType(int argc, char** argv)
{
   for ( int i(1); i < argc; ++i )
   {
      if ( std::strncmp(argv[i], "-i", 2) == 0 )
      {
         if ( i + 1 == argc )
         {
            std::string message = "Command line option -i needs a parameter:";
            message += " Choose either \"16\", \"32\", \"64\", \"safe\" or \"inf\"-";
            throw std::invalid_argument(message);
         }
         return integerTypeFromString(argv[i + 1]);
      }
      else if ( std::strncmp(argv[i], "--integer-type=", 15) == 0 )
      {
         return integerTypeFromString(argv[i] + 15);
      }
   }
   return IntegerType::Default;
}

namespace
{
   IntegerType integerTypeFromString(char* string)
   {
      if ( std::strcmp(string, "16") == 0 )
      {
         return IntegerType::Fixed16;
      }
      else if ( std::strcmp(string, "32") == 0 )
      {
         return IntegerType::Fixed32;
      }
      else if ( std::strcmp(string, "64") == 0 )
      {
         return IntegerType::Fixed64;
      }
      else if ( std::strcmp(string, "safe") == 0 )
      {
         return IntegerType::Safe;
      }
      else if ( std::strcmp(string, "inf") == 0 )
      {
         return IntegerType::Variable;
      }
      std::string message = "Invalid parameter to command line option -i";
      message += " / --integer-type: Choose either \"16\", \"32\", \"64\", \"safe\" or \"inf\"-";
      throw std::invalid_argument(message);
   }
}

