
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#define COMPILE_TEMPLATE_JOB_MANAGER_PROXY
#include "job_manager_proxy.h"
#undef COMPILE_TEMPLATE_JOB_MANAGER_PROXY

#ifdef MPI_SUPPORT

template <typename Integer>
void JobManagerProxy<Integer>::put(const Matrix<Integer>& container, FacetTag) const
{
   communication.toMaster(container);
}

template <typename Integer>
void JobManagerProxy<Integer>::put(const Matrix<Integer>& container, VertexTag) const
{
   communication.toMaster(container);
}

template <typename Integer>
Row<Integer> JobManagerProxy<Integer>::get() const
{
   return communication.fromMaster<Integer>();
}

#else

template <typename Integer>
void JobManagerProxy<Integer>::put(const Matrix<Integer>&, FacetTag) const
{
}

template <typename Integer>
void JobManagerProxy<Integer>::put(const Matrix<Integer>&, VertexTag) const
{
}

template <typename Integer>
Row<Integer> JobManagerProxy<Integer>::get() const
{
   return Row<Integer>{};
}

#endif

template <typename Integer>
JobManagerProxy<Integer>::JobManagerProxy(const Names&, const int, const int, FacetTag)
:
   communication()
{
}

template <typename Integer>
JobManagerProxy<Integer>::JobManagerProxy(const Names&, const int, const int, VertexTag)
:
   communication()
{
}

