
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#include "testing_gear.h"

#include "algorithm_integer_operations.h"

#include <limits>
#include <random>

int main()
try
{
   ASSERT(gcd(0, 15) == 15, "");
   ASSERT(gcd(15, 0) == 15, "");
   ASSERT(gcd(1, 15) == 1, "");
   ASSERT(gcd(15, 1) == 1, "");
   ASSERT(gcd(-36, 150) == 6, "");
   ASSERT(gcd(150, -36) == 6, "");
   std::random_device rd{};
   std::mt19937 mt{rd()};
   std::uniform_int_distribution<int> d_int(std::numeric_limits<int>::min(), std::numeric_limits<int>::max());
   for ( int i(0); i < 100; ++i )
   {
      const auto r0 = d_int(mt);
      const auto r1 = d_int(mt);
      ASSERT(gcd(r0, r1) == gcd(r1, r0), "gcd is symmetric");
      ASSERT((r0 == 0 && r1 == 0) || gcd(r0, r1) != 0, "gcd is not zero, if at least one number is not zero.");
      ASSERT(gcd(r0, r1) == 0 || (r0 % gcd(r0, r1)) == 0, "gcd != 0 must divide the numbers.");
      ASSERT(gcd(r0, r1) == 0 || (r1 % gcd(r0, r1)) == 0, "gcd != 0 must divide the numbers.");
   }
   #ifndef NO_FLEXIBILITY
   const auto max = std::numeric_limits<short>::max();
   const auto min = std::numeric_limits<short>::min();
   std::uniform_int_distribution<short> d_short(min, max);
   for ( int i(0); i < 100; ++i )
   {
      const auto r0 = d_short(mt);
      const auto r1 = d_short(mt);
      ASSERT(gcd(r0, r1) == gcd(r1, r0), "gcd is symmetric");
      ASSERT((r0 == 0 && r1 == 0) || gcd(r0, r1) != 0, "gcd is not zero, if at least one number is not zero.");
      ASSERT(gcd(r0, r1) == 0 || (r0 % gcd(r0, r1)) == 0, "gcd != 0 must divide the numbers.");
      ASSERT(gcd(r0, r1) == 0 || (r1 % gcd(r0, r1)) == 0, "gcd != 0 must divide the numbers.");
   }
   #endif
}
catch ( const std::exception& e )
{
   std::cerr << e.what() << "\n";
   return 1;
}

