
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#include "testing_gear.h"

#include "safe_integer.h"

namespace
{
   void test_operator_unary_minus();
   void test_abs();
}

int main()
try
{
   test_operator_unary_minus();
   test_abs();
}
catch ( const std::exception& e )
{
   std::cerr << e.what() << "\n";
   return 1;
}

namespace
{
   using SI = SafeInteger;
   using SID = SafeInteger::DataType;

   void test_operator_unary_minus()
   {
      ASSERT(-SI(0) == SI(0), "operator-()");
      ASSERT(-SI(1) == SI(-1), "operator-()");
      ASSERT(-SI(-1) == SI(1), "operator-()");
      if ( std::numeric_limits<SID>::min() + std::numeric_limits<SID>::max() < 0 )
      {
         ASSERT_ANY_EXCEPTION(-SI(std::numeric_limits<SID>::min()), "operator-() has to throw on invalid argument.");
      }
      else if ( std::numeric_limits<SID>::min() + std::numeric_limits<SID>::max() > 0 )
      {
         ASSERT_ANY_EXCEPTION(-SI(std::numeric_limits<SID>::max()), "operator-() has to throw on invalid argument.");
      }
   }

   void test_abs()
   {
      ASSERT(abs(SI(0)) == SI(0), "abs(SafeInteger)");
      ASSERT(abs(SI(1)) == SI(1), "abs(SafeInteger)");
      ASSERT(abs(SI(-1)) == SI(1), "abs(SafeInteger)");
      if ( std::numeric_limits<SID>::min() + std::numeric_limits<SID>::max() < 0 )
      {
         ASSERT_ANY_EXCEPTION(abs(SI(std::numeric_limits<SID>::min())), "abs(SafeInteger) has to throw on invalid argument.");
      }
      else if ( std::numeric_limits<SID>::min() + std::numeric_limits<SID>::max() > 0 )
      {
         ASSERT_NOTHROW(abs(SI(std::numeric_limits<SID>::max())), "abs(SafeInteger) may not throw on invalid argument.");
      }
   }
}

