
//-------------------------------------------------------------------------------//
// Author: Stefan Lörwald, Universität Heidelberg                                //
// License: CC BY-NC 4.0 http://creativecommons.org/licenses/by-nc/4.0/legalcode //
//-------------------------------------------------------------------------------//

#include "testing_gear.h"

#include "safe_integer.h"

namespace
{
   void test_operator_int_equality();
   void test_operator_int_inequality();
   void test_operator_int_less();
   void test_operator_int_greater();
   void test_operator_int_leq();
   void test_operator_int_geq();
   void test_operator_bigint_equality();
   void test_operator_bigint_inequality();
   void test_operator_bigint_less();
   void test_operator_bigint_greater();
   void test_operator_bigint_leq();
   void test_operator_bigint_geq();
}

int main()
try
{
   test_operator_int_equality();
   test_operator_int_inequality();
   test_operator_int_less();
   test_operator_int_greater();
   test_operator_int_leq();
   test_operator_int_geq();
   test_operator_bigint_equality();
   test_operator_bigint_inequality();
   test_operator_bigint_less();
   test_operator_bigint_greater();
   test_operator_bigint_leq();
   test_operator_bigint_geq();
}
catch ( const std::exception& e )
{
   std::cerr << e.what() << "\n";
   return 1;
}

namespace
{
   using SI = SafeInteger;

   void test_operator_int_equality()
   {
      ASSERT(SI(10) == 10, "operator==(int)");
      ASSERT(SI(-10) == -10, "operator==(int)");
   }

   void test_operator_int_inequality()
   {
      ASSERT(!(SI(10) != 10), "operator!=(int)");
      ASSERT(!(SI(-10) != -10), "operator!=(int)");
   }

   void test_operator_int_less()
   {
      ASSERT(SI(10) < 11, "operator<(int)");
      ASSERT(SI(-10) < -9, "operator<(int)");
   }

   void test_operator_int_greater()
   {
      ASSERT(SI(10) > 9, "operator>(int)");
      ASSERT(SI(-10) > -11, "operator>(int)");
   }

   void test_operator_int_leq()
   {
      ASSERT(SI(10) <= 10, "operator<=(int)");
      ASSERT(SI(10) <= 11, "operator<=(int)");
      ASSERT(SI(-10) <= -10, "operator<=(int)");
      ASSERT(SI(-10) <= -9, "operator<=(int)");
   }

   void test_operator_int_geq()
   {
      ASSERT(SI(10) >= 10, "operator>=(int)");
      ASSERT(SI(-10) >= -10, "operator>=(int)");
      ASSERT(SI(10) >= 9, "operator>=(int)");
      ASSERT(SI(-10) >= -11, "operator>=(int)");
   }

   void test_operator_bigint_equality()
   {
      ASSERT(SI(10) == SI(10), "operator==(SafeInteger)");
      ASSERT(SI(-10) == SI(-10), "operator==(SafeInteger)");
   }

   void test_operator_bigint_inequality()
   {
      ASSERT(!(SI(10) != SI(10)), "operator!=(SafeInteger)");
      ASSERT(!(SI(-10) != SI(-10)), "operator!=(SafeInteger)");
   }

   void test_operator_bigint_less()
   {
      ASSERT(SI(0) < SI(1), "operator<(SafeInteger)");
      ASSERT(SI(1) < SI(2), "operator<(SafeInteger)");
      ASSERT(SI(-1) < SI(2), "operator<(SafeInteger)");
      ASSERT(SI(-2) < SI(-1), "operator<(SafeInteger)");
      ASSERT(SI(-1) < SI(0), "operator<(SafeInteger)");
   }

   void test_operator_bigint_greater()
   {
      ASSERT(SI(1) > SI(0), "operator<(SafeInteger)");
      ASSERT(SI(2) > SI(1), "operator<(SafeInteger)");
      ASSERT(SI(2) > SI(-1), "operator<(SafeInteger)");
      ASSERT(SI(-1) > SI(-2), "operator<(SafeInteger)");
      ASSERT(SI(0) > SI(-1), "operator<(SafeInteger)");
   }

   void test_operator_bigint_leq()
   {
      ASSERT(SI(0) <= SI(1), "operator<(SafeInteger)");
      ASSERT(SI(0) <= SI(0), "operator<(SafeInteger)");
      ASSERT(SI(1) <= SI(2), "operator<(SafeInteger)");;
      ASSERT(SI(1) <= SI(1), "operator<(SafeInteger)");
      ASSERT(SI(-1) <= SI(2), "operator<(SafeInteger)");
      ASSERT(SI(-2) <= SI(-1), "operator<(SafeInteger)");
      ASSERT(SI(-1) <= SI(0), "operator<(SafeInteger)");
      ASSERT(SI(-1) <= SI(-1), "operator<(SafeInteger)");
   }

   void test_operator_bigint_geq()
   {
      ASSERT(SI(1) >= SI(0), "operator<(SafeInteger)");
      ASSERT(SI(0) >= SI(0), "operator<(SafeInteger)");
      ASSERT(SI(2) >= SI(1), "operator<(SafeInteger)");;
      ASSERT(SI(1) >= SI(1), "operator<(SafeInteger)");
      ASSERT(SI(2) >= SI(-1), "operator<(SafeInteger)");
      ASSERT(SI(-1) >= SI(-2), "operator<(SafeInteger)");
      ASSERT(SI(0) >= SI(-1), "operator<(SafeInteger)");
      ASSERT(SI(-1) >= SI(-1), "operator<(SafeInteger)");
   }
}

